package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.SysUser;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.model.TaVisitingLog;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.SMSUtils;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.*;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
@Api(value = "人员管理API", description = "人员管理的API")
public class TaPersonController extends BaseController {
    @Autowired
    private ITaPersonService taPersonService;

    @Autowired
    private ITaVisitingLogService taVisitingLogService;
    
    @Autowired
    ITaDeviceService taDeviceService;

    @Autowired
    private ICameraService cameraService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    SMSUtils smsUtils;

    @ApiOperation(value = "新增人员", notes = "新增人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "TaPerson", name = "taPerson", value = "人员详情")
    })
    @RequestMapping(value = "/person", method = RequestMethod.POST)
    public ResponseBean create(@RequestBody TaPerson taPerson, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        taPerson.setStatus(StatusUtils.Normal);
        boolean success = taPersonService.save(taPerson);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增人员失败");
            return responseBean;
        }

        taPerson.setRealId(taPerson.getPersonId());
        success = taPersonService.updateById(taPerson);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增人员失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "编辑人员", notes = "编辑人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "TaPerson", name = "taPerson", value = "人员详情")
    })
    @RequestMapping(value = "/person", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaPerson taPerson, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        boolean success = taPersonService.updateById(taPerson);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新人员失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "合并人员", notes = "合并人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "int", name = "from", value = "待合并人员ID"),
            @ApiImplicitParam(paramType = "body", dataType = "TaPerson", name = "to", value = "合并到人员ID")
    })
    @RequestMapping(value = "/merge/person/{from}", method = RequestMethod.PUT)
    public  ResponseBean merge(@PathVariable Integer from, @RequestBody TaPerson to) {
        ResponseBean responseBean = new ResponseBean();
        boolean success = taPersonService.mergePerson(from, to);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "合并人员失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "更新人员头像", notes = "更新人员头像")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="path", dataType="Integer", name="personId", value="人员ID"),
            @ApiImplicitParam(paramType="path", dataType="Integer", name="logId", value="访问日志ID"
            )}
    )
    @RequestMapping(value = "/person/{personId}/log/{logId}", method = RequestMethod.PUT)
    public ResponseBean updateAvatar(@PathVariable int personId, @PathVariable int logId, HttpServletRequest request) throws Exception {
        ResponseBean responseBean = new ResponseBean();

        TaVisitingLog taVisitingLog = taVisitingLogService.getById(logId);

        TaPerson taPerson = new TaPerson();
        taPerson.setPersonId(personId);
        taPerson.setAvatar(taVisitingLog.getAvatar());

        String success = taPersonService.updateAvatar(taPerson);
        if (!success.equals("success")) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, success);
            return responseBean;
        }
        return responseBean;
    }

    @ApiOperation(value = "下发人员数据", notes = "下发人员数据")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="path", dataType="Integer", name="personId", value="人员ID"),
    })
    @RequestMapping(value = "/person/{personId}/avatar", method = RequestMethod.PUT)
    public ResponseBean DispatchProfile(@PathVariable int personId, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        TaPerson taPerson = taPersonService.getById(personId);
        String result = cameraService.dispatchPerson(taPerson);
        boolean success = null == result || "".equals(result) || "success".equals(result);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, result);
            return responseBean;
        }
        return responseBean;
    }

	@ApiOperation(value = "获取人员详情", notes = "获取人员详情")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "path", dataType = "Integer", name = "id", value = "人员ID")
	})
	@RequestMapping(value = "/person/{id}", method = RequestMethod.GET)
	public ResponseBean get(@PathVariable int id, HttpServletRequest request) {
		ResponseBean responseBean = new ResponseBean();

//		Map<String, Object> res = taPersonService.detail(id);
        Map<String, Object> res = new HashMap<>();
        res.put("detail", taPersonService.getById(id));

		responseBean.addSuccess(res);
		return responseBean;
	}

    @ApiOperation(value = "获取人员列表", notes = "获取人员列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "typeId", value = "类型"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "personId", value = "人员ID"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "name", value = "姓名"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "phone", value = "电话"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "beginDate", value = "起始日期"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "endDate", value = "结束日期"),
    })
    @RequestMapping(value = "/person", method = RequestMethod.GET)
    public ResponseBean list(
            @RequestParam Integer pageNum,
            @RequestParam  Integer pageSize,
            Integer typeId,
            Integer personId,
            String name,
            String phone,
            String beginDate,
            String endDate,
            HttpServletRequest request
    ) {
        ResponseBean responseBean = new ResponseBean();

        Page<TaPerson> page = new Page<>(pageNum, pageSize);

        if (null != typeId && typeId < 1) typeId = null;
        if (null != personId && personId < 1) personId = null;
        if (null != name && "".equals(name)) name = null;
        if (null != phone && "".equals(phone)) phone = null;
        if (null != beginDate && "".equals(beginDate)) beginDate = null;
        if (null != endDate && "".equals(endDate)) endDate = null;

        responseBean.addSuccess(taPersonService.getRealPersonList(page, personId, typeId, name, phone, beginDate, endDate));

        return responseBean;
    }

    @ApiOperation(value = "统计人员", notes = "统计人员")
    @RequestMapping(value = "/stats/person", method = RequestMethod.GET)
    public ResponseBean statsPerson() {
        ResponseBean responseBean = new ResponseBean();

        List<Map<String, String>> person = taPersonService.statsPerson();
        if (null == person) {
            person = new ArrayList<>();
        }
        if (person.size() == 0) {
            Map<String, String> mp = new HashMap<>();
            person.add(mp);
        }

        Long regularClients = taVisitingLogService.getRegularClientTodayAmount();
        if (null ==  regularClients) regularClients = 0l;

        person.get(0).put("regular", String.valueOf(regularClients));

        responseBean.addSuccess(person);
        return responseBean;
    }

  @ApiOperation(value = "统计人员", notes = "统计人员")
  @RequestMapping(value = "/wx/stats/person", method = RequestMethod.GET)
  public ResponseBean statsWxPerson() {
    ResponseBean responseBean = new ResponseBean();

    List<Map<String, String>> person = taPersonService.statsPerson();
    if (null == person) {
      person = new ArrayList<>();
    }
    if (person.size() == 0) {
      Map<String, String> mp = new HashMap<>();
      person.add(mp);
    }

    Long regularClients = taVisitingLogService.getRegularClientTodayAmount();
    if (null ==  regularClients) regularClients = 0l;

    person.get(0).put("regular", String.valueOf(regularClients));

    responseBean.addSuccess(person);
    return responseBean;
  }

    @PutMapping("/wx/loginPhone")
    public ResponseBean loginPhone(HttpServletRequest request,@RequestParam String phone,@RequestParam String code,@RequestParam String openId) {
        ResponseBean responseBean = new ResponseBean();
        HttpSession httpSession = request.getSession();
        if(!smsUtils.checkCaptcha(phone,code)){
            responseBean.addError("验证码错误。");
            return responseBean;
        }
        QueryWrapper<SysUser> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("phone",phone);
        userQueryWrapper.eq("status",1);
        userQueryWrapper.eq("is_wechat_admin",1);
        List<SysUser>personList = sysUserService.list(userQueryWrapper);
        if (personList.size() <= 0){
            responseBean.addError("未知手机号，请联系系统管理员。");
        }
        SysUser sysUser = personList.get(0);
        sysUser.setOpenid(openId);
        sysUserService.updateById(sysUser);
        responseBean.addSuccess(sysUser);
        httpSession.setAttribute("token", sysUser.getUsername());
        httpSession.setAttribute("token-id", sysUser.getUserId());
        responseBean.addSuccess("登录成功");
        return responseBean;
    }

    /**
     * 发送HttpGet请求 openid
     * @param code
     * @return
     */
    @GetMapping("/wx/getOpenId")
    public String getOpenId(String code) {
        //1.获得一个httpclient对象
        CloseableHttpClient httpclient = HttpClients.createDefault();
        Map<String,String> map =new HashMap<>();
        map.put("appid","wx18765573b7565aed");
        map.put("code",code);
        List<NameValuePair> formparams = new ArrayList<NameValuePair>();
        //2.生成一个get请求
        for (Map.Entry<String, String> entry : map.entrySet()) {
            //给参数赋值
            formparams.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
        }
        String url = "http://47.101.36.130:9000/wx/userinfo";
        UrlEncodedFormEntity entity = new UrlEncodedFormEntity(formparams, Consts.UTF_8);
        HttpPost httppost = new HttpPost(url);
        httppost.setEntity(entity);

        CloseableHttpResponse response = null;
        try {
            response = httpclient.execute(httppost);
        } catch (IOException e) {
            e.printStackTrace();
        }
        HttpEntity entity1 = response.getEntity();
        String result = null;
        try {
            result = EntityUtils.toString(entity1);
        } catch (ParseException | IOException e) {
            e.printStackTrace();
        }
        return result;
    }
    /**
     * 发送HttpGet请求 openid
     * @param openid
     * @return
     */
    @GetMapping("/wx/loginByOpenid")
    public ResponseBean checkPersonByOpenId(@RequestParam String openid,HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        HttpSession httpSession = request.getSession();
        QueryWrapper<SysUser>queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id",openid);
        queryWrapper.eq("status",1);
        queryWrapper.eq("is_wechat_admin",1);
        try{
            SysUser user = sysUserService.getOne(queryWrapper);
            if(null != user){
                httpSession.setAttribute("token", user.getUsername());
                httpSession.setAttribute("token-id", user.getUserId());
                responseBean.addSuccess(user);
            }else {
                responseBean.addError("未获取到用户信息，请登录或联系管理员");
            }
        }catch (Exception e){
            e.printStackTrace();
            responseBean.addError("程序错误，请稍后再试");
        }
        return responseBean;
    }

    /**
     * 发送HttpGet请求 openid
     * @param openid
     * @return
     */
    @GetMapping("/wx/getUserInfo")
    public ResponseBean getUserInfo(@RequestParam String openid){
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<SysUser>queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id",openid);
        queryWrapper.eq("status",1);
        try{
            SysUser user = sysUserService.getOne(queryWrapper);
            if(null != user){
                responseBean.addSuccess(user);
            }else {
                responseBean.addError("未获取到用户信息，请登录");
            }
        }catch (Exception e){
            e.printStackTrace();
            responseBean.addError("程序错误，请稍后再试");
        }
        return responseBean;
    }

    @PostMapping("/wx/captcha")
    public ResponseBean sendCaptcha(@RequestParam String tel) {
        ResponseBean responseBean = new ResponseBean();
        boolean success = smsUtils.sendCaptcha(tel);
        if (success) {
            responseBean.addSuccess("发送验证码成功");
            return responseBean;
        } else {
            responseBean.addError(0,"验证码发送失败, 请重试");
            return responseBean;
        }
    }

}
