package com.huiju.welcome.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaSnapBackground;
import com.huiju.welcome.service.ITaSnapBackgroundService;
import com.huiju.welcome.utils.XFYun;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import com.huiju.welcome.common.base.BaseController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-11-28
 */
@RestController
@RequestMapping("/")
public class TaSnapBackgroundController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaSnapBackgroundController.class);
    private Logger log = LoggerFactory.getLogger(XFYun.class);

    @Value("${huiju.files}")
    private String huijuFiles;
    @Autowired
    public ITaSnapBackgroundService iTaSnapBackgroundService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taSnapBackground",method= RequestMethod.GET)
    public ResponseBean taSnapBackgroundList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaSnapBackground> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaSnapBackground> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaSnapBackground> result = iTaSnapBackgroundService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            logger.error("taSnapBackgroundList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taSnapBackground 实体对象
     * @return
     */
    @ApiOperation(value = "人脸对比结果上传", notes = "人脸对比结果上传")
    @PostMapping(value = "/camera/{deviceId}/taSnapBackground")
    public ResponseBean taSnapBackgroundAdd(
            @PathVariable String deviceId,
            MultipartFile backgroundFile,
            @RequestBody TaSnapBackground taSnapBackground){
        ResponseBean responseBean = new ResponseBean();
        try {
            long nowTime = System.currentTimeMillis();
            String avatar = "/camera/" + nowTime + "_background_" + taSnapBackground.getFaceId() +"device"+deviceId+ ".jpg";
            getMultipartFile(backgroundFile, avatar);
            taSnapBackground.setBackground(avatar);
            if (iTaSnapBackgroundService.save(taSnapBackground)){
                responseBean.addSuccess(taSnapBackground);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taSnapBackgroundAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taSnapBackground/{id}", method= RequestMethod.DELETE)
    public ResponseBean taSnapBackgroundDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaSnapBackgroundService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taSnapBackgroundDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taSnapBackground 实体对象
     * @return
     */
    @RequestMapping(value="/taSnapBackground/{id}",method= RequestMethod.PUT)
    public ResponseBean taSnapBackgroundUpdate(@PathVariable Integer id,
                                        @RequestBody TaSnapBackground taSnapBackground){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaSnapBackgroundService.updateById(taSnapBackground)){
                responseBean.addSuccess(taSnapBackground);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taSnapBackgroundUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taSnapBackground/{id}",method= RequestMethod.GET)
    public ResponseBean taSnapBackgroundGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaSnapBackgroundService.getById(id));
        }catch (Exception e){
            logger.error("taSnapBackgroundDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    // 获取上传单文件
    private boolean getMultipartFile(MultipartFile f, String newFname) {
        if (null == f || f.isEmpty()) {
            log.error("========================没有获取到背景图片");
            return false;
        } else {
            log.info("=======================成功获取到背景数据流");
        }

        File localDir = new File(huijuFiles + "camera");
        if (!localDir.exists()) localDir.mkdirs();

        try {
            byte[] fBytes = f.getBytes();
            ByteBuffer buf = ByteBuffer.wrap(fBytes);
            FileChannel fc = new FileOutputStream(huijuFiles + newFname).getChannel();
            fc.write(buf);
            fc.close();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("读取背景图片失败: " + e.getMessage());
            return false;
        }

        log.info("成功落地背景图: " + huijuFiles + newFname);

        return true;
    }
}
